package org.eclipse.jdt.internal.compiler.apt.model;

import org.eclipse.jdt.internal.compiler.lookup.ElementValuePair;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import javax.lang.model.type.ErrorType;
import javax.lang.model.element.TypeParameterElement;
import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import javax.lang.model.type.NoType;
import javax.lang.model.type.TypeKind;
import javax.lang.model.type.NullType;
import javax.lang.model.element.PackageElement;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;
import javax.lang.model.element.Element;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import java.util.EnumSet;
import javax.lang.model.element.ElementKind;
import javax.lang.model.type.TypeMirror;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import javax.lang.model.element.Modifier;
import java.util.Set;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.core.compiler.CharOperation;
import java.lang.reflect.Proxy;
import java.lang.reflect.Array;
import java.lang.annotation.Annotation;
import java.util.ArrayList;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import java.util.Collections;
import javax.lang.model.element.AnnotationMirror;
import java.util.List;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseProcessingEnvImpl;

public class Factory
{
    public static final Byte DUMMY_BYTE;
    public static final Character DUMMY_CHAR;
    public static final Double DUMMY_DOUBLE;
    public static final Float DUMMY_FLOAT;
    public static final Integer DUMMY_INTEGER;
    public static final Long DUMMY_LONG;
    public static final Short DUMMY_SHORT;
    private final BaseProcessingEnvImpl _env;
    public static List<? extends AnnotationMirror> EMPTY_ANNOTATION_MIRRORS;
    private static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$element$ElementKind;
    private static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$type$TypeKind;
    
    static {
        DUMMY_BYTE = Byte.valueOf((byte)0);
        DUMMY_CHAR = Character.valueOf('0');
        DUMMY_DOUBLE = Double.valueOf(0.0);
        DUMMY_FLOAT = Float.valueOf(0.0f);
        DUMMY_INTEGER = Integer.valueOf(0);
        DUMMY_LONG = Long.valueOf(0L);
        DUMMY_SHORT = Short.valueOf((short)0);
        Factory.EMPTY_ANNOTATION_MIRRORS = Collections.emptyList();
    }
    
    public Factory(BaseProcessingEnvImpl env) {
        this._env = env;
    }
    
    public List<? extends AnnotationMirror> getAnnotationMirrors(AnnotationBinding[] annotations) {
        if (annotations == null || annotations.length == 0) {
            return Collections.emptyList();
        }
        List<AnnotationMirror> list = new ArrayList(annotations.length);
        for (AnnotationBinding annotation : annotations) {
            if (annotation != null) {
                list.add(this.newAnnotationMirror(annotation));
            }
        }
        return Collections.unmodifiableList(list);
    }
    
    public <A extends Annotation> A[] getAnnotationsByType(AnnotationBinding[] annoInstances, Class<A> annotationClass) {
        Annotation[] result = this.getAnnotations(annoInstances, annotationClass, false);
        return (result == null) ? ((Annotation[])Array.newInstance(annotationClass, 0)) : result;
    }
    
    public <A extends Annotation> A getAnnotation(AnnotationBinding[] annoInstances, Class<A> annotationClass) {
        Annotation[] result = this.getAnnotations(annoInstances, annotationClass, true);
        return (result == null) ? null : result[0];
    }
    
    private <A extends Annotation> A[] getAnnotations(AnnotationBinding[] annoInstances, Class<A> annotationClass, boolean justTheFirst) {
        if (annoInstances == null || annoInstances.length == 0 || annotationClass == null) {
            return null;
        }
        String annoTypeName = annotationClass.getName();
        if (annoTypeName == null) {
            return null;
        }
        List<A> list = new ArrayList(annoInstances.length);
        for (AnnotationBinding annoInstance : annoInstances) {
            if (annoInstance != null) {
                AnnotationMirrorImpl annoMirror = this.createAnnotationMirror(annoTypeName, annoInstance);
                if (annoMirror != null) {
                    list.add((Object)Proxy.newProxyInstance(annotationClass.getClassLoader(), new Class[] { annotationClass }, annoMirror));
                    if (justTheFirst) {
                        break;
                    }
                }
            }
        }
        Annotation[] result = (Annotation[])Array.newInstance(annotationClass, list.size());
        return (list.size() > 0) ? ((A[])list.toArray(result)) : null;
    }
    
    private AnnotationMirrorImpl createAnnotationMirror(String annoTypeName, AnnotationBinding annoInstance) {
        ReferenceBinding binding = annoInstance.getAnnotationType();
        if (binding != null && binding.isAnnotationType()) {
            char[] qName;
            if (binding.isMemberType()) {
                annoTypeName = annoTypeName.replace('$', '.');
                qName = CharOperation.concatWith(binding.enclosingType().compoundName, binding.sourceName, '.');
                CharOperation.replace(qName, '$', '.');
            }
            else {
                qName = CharOperation.concatWith(binding.compoundName, '.');
            }
            if (annoTypeName.equals(new String(qName))) {
                return (AnnotationMirrorImpl)this._env.getFactory().newAnnotationMirror(annoInstance);
            }
        }
        return null;
    }
    
    private static void appendModifier(Set<Modifier> result, int modifiers, int modifierConstant, Modifier modifier) {
        if ((modifiers & modifierConstant) != 0x0) {
            result.add(modifier);
        }
    }
    
    private static void decodeModifiers(Set<Modifier> result, int modifiers, int[] checkBits) {
        if (checkBits == null) {
            return;
        }
        for (int i = 0, max = checkBits.length; i < max; ++i) {
            switch (checkBits[i]) {
                case 1: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.PUBLIC);
                    break;
                }
                case 4: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.PROTECTED);
                    break;
                }
                case 2: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.PRIVATE);
                    break;
                }
                case 1024: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.ABSTRACT);
                    break;
                }
                case 65536: {
                    try {
                        appendModifier(result, modifiers, checkBits[i], Modifier.valueOf("DEFAULT"));
                        break;
                    }
                    catch (IllegalArgumentException ex) {
                        break;
                    }
                }
                case 8: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.STATIC);
                    break;
                }
                case 16: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.FINAL);
                    break;
                }
                case 32: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.SYNCHRONIZED);
                    break;
                }
                case 256: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.NATIVE);
                    break;
                }
                case 2048: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.STRICTFP);
                    break;
                }
                case 128: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.TRANSIENT);
                    break;
                }
                case 64: {
                    appendModifier(result, modifiers, checkBits[i], Modifier.VOLATILE);
                    break;
                }
            }
        }
        return;
    }
    
    public static Object getMatchingDummyValue(Class<?> expectedType) {
        if (expectedType.isPrimitive()) {
            if (expectedType == Boolean.TYPE) {
                return Boolean.FALSE;
            }
            if (expectedType == Byte.TYPE) {
                return Factory.DUMMY_BYTE;
            }
            if (expectedType == Character.TYPE) {
                return Factory.DUMMY_CHAR;
            }
            if (expectedType == Double.TYPE) {
                return Factory.DUMMY_DOUBLE;
            }
            if (expectedType == Float.TYPE) {
                return Factory.DUMMY_FLOAT;
            }
            if (expectedType == Integer.TYPE) {
                return Factory.DUMMY_INTEGER;
            }
            if (expectedType == Long.TYPE) {
                return Factory.DUMMY_LONG;
            }
            if (expectedType == Short.TYPE) {
                return Factory.DUMMY_SHORT;
            }
            return Factory.DUMMY_INTEGER;
        }
        else {
            return null;
        }
    }
    
    public TypeMirror getReceiverType(MethodBinding binding) {
        if (binding != null) {
            if (binding.receiver != null) {
                return this._env.getFactory().newTypeMirror(binding.receiver);
            }
            if (binding.declaringClass != null && !binding.isStatic() && (!binding.isConstructor() || binding.declaringClass.isMemberType())) {
                return this._env.getFactory().newTypeMirror(binding.declaringClass);
            }
        }
        return NoTypeImpl.NO_TYPE_NONE;
    }
    
    public static Set<Modifier> getModifiers(int modifiers, ElementKind kind) {
        return getModifiers(modifiers, kind, false);
    }
    
    public static Set<Modifier> getModifiers(int modifiers, ElementKind kind, boolean isFromBinary) {
        EnumSet<Modifier> result = EnumSet.noneOf(Modifier.class);
        switch ($SWITCH_TABLE$javax$lang$model$element$ElementKind()[kind.ordinal()]) {
            case 11:
            case 12: {
                decodeModifiers(result, modifiers, new int[] { 1, 4, 2, 1024, 8, 16, 32, 256, 2048, 65536 });
                break;
            }
            case 6:
            case 7: {
                decodeModifiers(result, modifiers, new int[] { 1, 4, 2, 8, 16, 128, 64 });
                break;
            }
            case 2: {
                if (isFromBinary) {
                    decodeModifiers(result, modifiers, new int[] { 1, 4, 16, 2, 1024, 8, 2048 });
                    break;
                }
                decodeModifiers(result, modifiers, new int[] { 1, 4, 16, 2, 8, 2048 });
                break;
            }
            case 3:
            case 4:
            case 5: {
                decodeModifiers(result, modifiers, new int[] { 1, 4, 1024, 16, 2, 8, 2048 });
                break;
            }
        }
        return Collections.unmodifiableSet(result);
    }
    
    public AnnotationMirror newAnnotationMirror(AnnotationBinding binding) {
        return new AnnotationMirrorImpl(this._env, binding);
    }
    
    public Element newElement(Binding binding, ElementKind kindHint) {
        if (binding == null) {
            return null;
        }
        switch (binding.kind()) {
            case 1:
            case 2:
            case 3: {
                return new VariableElementImpl(this._env, (VariableBinding)binding);
            }
            case 4:
            case 2052: {
                ReferenceBinding referenceBinding = (ReferenceBinding)binding;
                if ((referenceBinding.tagBits & 0x80L) != 0x0L) {
                    return new ErrorTypeElement(this._env, referenceBinding);
                }
                if (CharOperation.equals(referenceBinding.sourceName, TypeConstants.PACKAGE_INFO_NAME)) {
                    return new PackageElementImpl(this._env, referenceBinding.fPackage);
                }
                return new TypeElementImpl(this._env, referenceBinding, kindHint);
            }
            case 8: {
                return new ExecutableElementImpl(this._env, (MethodBinding)binding);
            }
            case 260:
            case 1028: {
                return new TypeElementImpl(this._env, ((ParameterizedTypeBinding)binding).genericType(), kindHint);
            }
            case 16: {
                return new PackageElementImpl(this._env, (PackageBinding)binding);
            }
            case 4100: {
                return new TypeParameterElementImpl(this._env, (TypeVariableBinding)binding);
            }
            case 32:
            case 68:
            case 132:
            case 516:
            case 8196: {
                throw new UnsupportedOperationException("NYI: binding type " + binding.kind());
            }
            default: {
                return null;
            }
        }
    }
    
    public Element newElement(Binding binding) {
        return this.newElement(binding, null);
    }
    
    public PackageElement newPackageElement(PackageBinding binding) {
        return new PackageElementImpl(this._env, binding);
    }
    
    public NullType getNullType() {
        return NoTypeImpl.NULL_TYPE;
    }
    
    public NoType getNoType(TypeKind kind) {
        switch ($SWITCH_TABLE$javax$lang$model$type$TypeKind()[kind.ordinal()]) {
            case 10: {
                return NoTypeImpl.NO_TYPE_NONE;
            }
            case 9: {
                return NoTypeImpl.NO_TYPE_VOID;
            }
            case 17: {
                return NoTypeImpl.NO_TYPE_PACKAGE;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    public PrimitiveTypeImpl getPrimitiveType(TypeKind kind) {
        switch ($SWITCH_TABLE$javax$lang$model$type$TypeKind()[kind.ordinal()]) {
            case 1: {
                return PrimitiveTypeImpl.BOOLEAN;
            }
            case 2: {
                return PrimitiveTypeImpl.BYTE;
            }
            case 6: {
                return PrimitiveTypeImpl.CHAR;
            }
            case 8: {
                return PrimitiveTypeImpl.DOUBLE;
            }
            case 7: {
                return PrimitiveTypeImpl.FLOAT;
            }
            case 4: {
                return PrimitiveTypeImpl.INT;
            }
            case 5: {
                return PrimitiveTypeImpl.LONG;
            }
            case 3: {
                return PrimitiveTypeImpl.SHORT;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    public PrimitiveTypeImpl getPrimitiveType(BaseTypeBinding binding) {
        AnnotationBinding[] annotations = binding.getTypeAnnotations();
        if (annotations == null || annotations.length == 0) {
            return this.getPrimitiveType(PrimitiveTypeImpl.getKind(binding));
        }
        return new PrimitiveTypeImpl(this._env, binding);
    }
    
    public TypeMirror newTypeMirror(Binding binding) {
        switch (binding.kind()) {
            case 1:
            case 2:
            case 3: {
                return this.newTypeMirror(((VariableBinding)binding).type);
            }
            case 16: {
                return this.getNoType(TypeKind.PACKAGE);
            }
            case 32: {
                throw new UnsupportedOperationException("NYI: import type " + binding.kind());
            }
            case 8: {
                return new ExecutableTypeImpl(this._env, (MethodBinding)binding);
            }
            case 4:
            case 260:
            case 1028:
            case 2052: {
                ReferenceBinding referenceBinding = (ReferenceBinding)binding;
                if ((referenceBinding.tagBits & 0x80L) != 0x0L) {
                    return this.getErrorType(referenceBinding);
                }
                return new DeclaredTypeImpl(this._env, (ReferenceBinding)binding);
            }
            case 68: {
                return new ArrayTypeImpl(this._env, (ArrayBinding)binding);
            }
            case 132: {
                BaseTypeBinding btb = (BaseTypeBinding)binding;
                switch (btb.id) {
                    case 6: {
                        return this.getNoType(TypeKind.VOID);
                    }
                    case 12: {
                        return this.getNullType();
                    }
                    default: {
                        return this.getPrimitiveType(btb);
                    }
                }
            }
            case 516:
            case 8196: {
                return new WildcardTypeImpl(this._env, (WildcardBinding)binding);
            }
            case 4100: {
                return new TypeVariableImpl(this._env, (TypeVariableBinding)binding);
            }
            default: {
                return null;
            }
        }
    }
    
    public TypeParameterElement newTypeParameterElement(TypeVariableBinding variable, Element declaringElement) {
        return new TypeParameterElementImpl(this._env, variable, declaringElement);
    }
    
    public ErrorType getErrorType(ReferenceBinding binding) {
        return new ErrorTypeImpl(this._env, binding);
    }
    
    public static Object performNecessaryPrimitiveTypeConversion(Class<?> expectedType, Object value, boolean avoidReflectException) {
        assert !(!expectedType.isPrimitive()) : "expectedType is not a primitive type: " + expectedType.getName();
        if (value == null) {
            return avoidReflectException ? getMatchingDummyValue(expectedType) : null;
        }
        String typeName = expectedType.getName();
        char expectedTypeChar = typeName.charAt(0);
        int nameLen = typeName.length();
        if ((value instanceof Byte)) {
            byte b = (byte)value;
            switch (expectedTypeChar) {
                case 'b': {
                    if (nameLen == 4) {
                        return value;
                    }
                    return avoidReflectException ? Boolean.FALSE : value;
                }
                case 'c': {
                    return Character.valueOf((char)b);
                }
                case 'd': {
                    return new Double((double)b);
                }
                case 'f': {
                    return new Float((float)b);
                }
                case 'i': {
                    return Integer.valueOf((int)b);
                }
                case 'l': {
                    return Long.valueOf((long)b);
                }
                case 's': {
                    return Short.valueOf((short)b);
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Short)) {
            short s = (short)value;
            switch (expectedTypeChar) {
                case 'b': {
                    if (nameLen == 4) {
                        return Byte.valueOf((byte)s);
                    }
                    return avoidReflectException ? Boolean.FALSE : value;
                }
                case 'c': {
                    return Character.valueOf((char)s);
                }
                case 'd': {
                    return new Double((double)s);
                }
                case 'f': {
                    return new Float((float)s);
                }
                case 'i': {
                    return Integer.valueOf((int)s);
                }
                case 'l': {
                    return Long.valueOf((long)s);
                }
                case 's': {
                    return value;
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Character)) {
            char c = (char)value;
            switch (expectedTypeChar) {
                case 'b': {
                    if (nameLen == 4) {
                        return Byte.valueOf((byte)c);
                    }
                    return avoidReflectException ? Boolean.FALSE : value;
                }
                case 'c': {
                    return value;
                }
                case 'd': {
                    return new Double((double)c);
                }
                case 'f': {
                    return new Float((float)c);
                }
                case 'i': {
                    return Integer.valueOf((int)c);
                }
                case 'l': {
                    return Long.valueOf((long)c);
                }
                case 's': {
                    return Short.valueOf((short)c);
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Integer)) {
            int i = (int)value;
            switch (expectedTypeChar) {
                case 'b': {
                    if (nameLen == 4) {
                        return Byte.valueOf((byte)i);
                    }
                    return avoidReflectException ? Boolean.FALSE : value;
                }
                case 'c': {
                    return Character.valueOf((char)i);
                }
                case 'd': {
                    return new Double((double)i);
                }
                case 'f': {
                    return new Float((float)i);
                }
                case 'i': {
                    return value;
                }
                case 'l': {
                    return Long.valueOf((long)i);
                }
                case 's': {
                    return Short.valueOf((short)i);
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Long)) {
            long l = (long)value;
            switch (expectedTypeChar) {
                case 'b':
                case 'c':
                case 'i':
                case 's': {
                    return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
                }
                case 'd': {
                    return new Double((double)l);
                }
                case 'f': {
                    return new Float((float)l);
                }
                case 'l': {
                    return value;
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Float)) {
            float f = (float)value;
            switch (expectedTypeChar) {
                case 'b':
                case 'c':
                case 'i':
                case 'l':
                case 's': {
                    return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
                }
                case 'd': {
                    return new Double((double)f);
                }
                case 'f': {
                    return value;
                }
                default: {
                    throw new IllegalStateException("unknown type " + expectedTypeChar);
                }
            }
        }
        else if ((value instanceof Double)) {
            if (expectedTypeChar == 'd') {
                return value;
            }
            return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
        }
        else if ((value instanceof Boolean)) {
            if (expectedTypeChar == 'b' && nameLen == 7) {
                return value;
            }
            return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
        }
        else {
            return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
        }
    }
    
    public static void setArrayMatchingDummyValue(Object array, int i, Class<?> expectedLeafType) {
        if (Boolean.TYPE.equals(expectedLeafType)) {
            Array.setBoolean(array, i, false);
        }
        else if (Byte.TYPE.equals(expectedLeafType)) {
            Array.setByte(array, i, (byte)Factory.DUMMY_BYTE);
        }
        else if (Character.TYPE.equals(expectedLeafType)) {
            Array.setChar(array, i, (char)Factory.DUMMY_CHAR);
        }
        else if (Double.TYPE.equals(expectedLeafType)) {
            Array.setDouble(array, i, (double)Factory.DUMMY_DOUBLE);
        }
        else if (Float.TYPE.equals(expectedLeafType)) {
            Array.setFloat(array, i, (float)Factory.DUMMY_FLOAT);
        }
        else if (Integer.TYPE.equals(expectedLeafType)) {
            Array.setInt(array, i, (int)Factory.DUMMY_INTEGER);
        }
        else if (Long.TYPE.equals(expectedLeafType)) {
            Array.setLong(array, i, (long)Factory.DUMMY_LONG);
        }
        else if (Short.TYPE.equals(expectedLeafType)) {
            Array.setShort(array, i, (short)Factory.DUMMY_SHORT);
        }
        else {
            Array.set(array, i, null);
        }
    }
    
    public static AnnotationBinding[] getPackedAnnotationBindings(AnnotationBinding[] annotations) {
        int length = (annotations == null) ? 0 : annotations.length;
        if (length == 0) {
            return annotations;
        }
        AnnotationBinding[] repackagedBindings = annotations;
        for (int i = 0; i < length; ++i) {
            AnnotationBinding annotation = repackagedBindings[i];
            if (annotation != null) {
                ReferenceBinding annotationType = annotation.getAnnotationType();
                if (annotationType.isRepeatableAnnotationType()) {
                    ReferenceBinding containerType = annotationType.containerAnnotationType();
                    if (containerType != null) {
                        MethodBinding[] values = containerType.getMethods(TypeConstants.VALUE);
                        if (values != null && values.length == 1) {
                            MethodBinding value = values[0];
                            if (value.returnType != null && value.returnType.dimensions() == 1 && !TypeBinding.notEquals(value.returnType.leafComponentType(), annotationType)) {
                                List<AnnotationBinding> containees = null;
                                for (int j = i + 1; j < length; ++j) {
                                    AnnotationBinding otherAnnotation = repackagedBindings[j];
                                    if (otherAnnotation != null && otherAnnotation.getAnnotationType() == annotationType) {
                                        if (repackagedBindings == annotations) {
                                            System.arraycopy(repackagedBindings, 0, repackagedBindings = new AnnotationBinding[length], 0, length);
                                        }
                                        repackagedBindings[j] = null;
                                        if (containees == null) {
                                            containees = new ArrayList();
                                            containees.add(annotation);
                                        }
                                        containees.add(otherAnnotation);
                                    }
                                }
                                if (containees != null) {
                                    ElementValuePair[] elementValuePairs = { new ElementValuePair(TypeConstants.VALUE, containees.toArray(), value) };
                                    repackagedBindings[i] = new AnnotationBinding(containerType, elementValuePairs);
                                }
                            }
                        }
                    }
                }
            }
        }
        int finalTally = 0;
        for (int k = 0; k < length; ++k) {
            if (repackagedBindings[k] != null) {
                ++finalTally;
            }
        }
        if (repackagedBindings == annotations && finalTally == length) {
            return annotations;
        }
        annotations = new AnnotationBinding[finalTally];
        int k = 0;
        int l = 0;
        while (k < length) {
            if (repackagedBindings[k] != null) {
                annotations[l++] = repackagedBindings[k];
            }
            ++k;
        }
        return annotations;
    }
    
    public static AnnotationBinding[] getUnpackedAnnotationBindings(AnnotationBinding[] annotations) {
        int length = (annotations == null) ? 0 : annotations.length;
        if (length == 0) {
            return annotations;
        }
        List<AnnotationBinding> unpackedAnnotations = new ArrayList();
        for (AnnotationBinding annotation : annotations) {
            if (annotation != null) {
                unpackedAnnotations.add(annotation);
                ReferenceBinding annotationType = annotation.getAnnotationType();
                MethodBinding[] values = annotationType.getMethods(TypeConstants.VALUE);
                if (values != null && values.length == 1) {
                    MethodBinding value = values[0];
                    if (value.returnType.dimensions() == 1) {
                        TypeBinding containeeType = value.returnType.leafComponentType();
                        if (containeeType != null && containeeType.isAnnotationType() && containeeType.isRepeatableAnnotationType() && containeeType.containerAnnotationType() == annotationType) {
                            ElementValuePair[] elementValuePairs = annotation.getElementValuePairs();
                            int length2 = elementValuePairs.length;
                            int j = 0;
                            while (j < length2) {
                                ElementValuePair elementValuePair = elementValuePairs[j];
                                if (CharOperation.equals(elementValuePair.getName(), TypeConstants.VALUE)) {
                                    for (Object object : (Object[])elementValuePair.getValue()) {
                                        unpackedAnnotations.add((AnnotationBinding)object);
                                    }
                                    break;
                                }
                                ++j;
                            }
                        }
                    }
                }
            }
        }
        return (AnnotationBinding[])unpackedAnnotations.toArray(new AnnotationBinding[unpackedAnnotations.size()]);
    }
    
    static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$element$ElementKind() {
        int[] $switch_TABLE$javax$lang$model$element$ElementKind = Factory.$SWITCH_TABLE$javax$lang$model$element$ElementKind;
        if ($switch_TABLE$javax$lang$model$element$ElementKind != null) {
            return $switch_TABLE$javax$lang$model$element$ElementKind;
        }
        int[] $switch_TABLE$javax$lang$model$element$ElementKind2 = new int[ElementKind.values().length];
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ANNOTATION_TYPE.ordinal()] = 4;
        }
        catch (NoSuchFieldError noSuchFieldError) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.CLASS.ordinal()] = 3;
        }
        catch (NoSuchFieldError noSuchFieldError2) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.CONSTRUCTOR.ordinal()] = 12;
        }
        catch (NoSuchFieldError noSuchFieldError3) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ENUM.ordinal()] = 2;
        }
        catch (NoSuchFieldError noSuchFieldError4) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.ENUM_CONSTANT.ordinal()] = 6;
        }
        catch (NoSuchFieldError noSuchFieldError5) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.EXCEPTION_PARAMETER.ordinal()] = 10;
        }
        catch (NoSuchFieldError noSuchFieldError6) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.FIELD.ordinal()] = 7;
        }
        catch (NoSuchFieldError noSuchFieldError7) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.INSTANCE_INIT.ordinal()] = 14;
        }
        catch (NoSuchFieldError noSuchFieldError8) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.INTERFACE.ordinal()] = 5;
        }
        catch (NoSuchFieldError noSuchFieldError9) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.LOCAL_VARIABLE.ordinal()] = 9;
        }
        catch (NoSuchFieldError noSuchFieldError10) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.METHOD.ordinal()] = 11;
        }
        catch (NoSuchFieldError noSuchFieldError11) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.OTHER.ordinal()] = 16;
        }
        catch (NoSuchFieldError noSuchFieldError12) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.PACKAGE.ordinal()] = 1;
        }
        catch (NoSuchFieldError noSuchFieldError13) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.PARAMETER.ordinal()] = 8;
        }
        catch (NoSuchFieldError noSuchFieldError14) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.RESOURCE_VARIABLE.ordinal()] = 17;
        }
        catch (NoSuchFieldError noSuchFieldError15) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.STATIC_INIT.ordinal()] = 13;
        }
        catch (NoSuchFieldError noSuchFieldError16) {}
        try {
            $switch_TABLE$javax$lang$model$element$ElementKind2[ElementKind.TYPE_PARAMETER.ordinal()] = 15;
        }
        catch (NoSuchFieldError noSuchFieldError17) {}
        return Factory.$SWITCH_TABLE$javax$lang$model$element$ElementKind = $switch_TABLE$javax$lang$model$element$ElementKind2;
    }
    
    static /* synthetic */ int[] $SWITCH_TABLE$javax$lang$model$type$TypeKind() {
        int[] $switch_TABLE$javax$lang$model$type$TypeKind = Factory.$SWITCH_TABLE$javax$lang$model$type$TypeKind;
        if ($switch_TABLE$javax$lang$model$type$TypeKind != null) {
            return $switch_TABLE$javax$lang$model$type$TypeKind;
        }
        int[] $switch_TABLE$javax$lang$model$type$TypeKind2 = new int[TypeKind.values().length];
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.ARRAY.ordinal()] = 12;
        }
        catch (NoSuchFieldError noSuchFieldError) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.BOOLEAN.ordinal()] = 1;
        }
        catch (NoSuchFieldError noSuchFieldError2) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.BYTE.ordinal()] = 2;
        }
        catch (NoSuchFieldError noSuchFieldError3) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.CHAR.ordinal()] = 6;
        }
        catch (NoSuchFieldError noSuchFieldError4) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.DECLARED.ordinal()] = 13;
        }
        catch (NoSuchFieldError noSuchFieldError5) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.DOUBLE.ordinal()] = 8;
        }
        catch (NoSuchFieldError noSuchFieldError6) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.ERROR.ordinal()] = 14;
        }
        catch (NoSuchFieldError noSuchFieldError7) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.EXECUTABLE.ordinal()] = 18;
        }
        catch (NoSuchFieldError noSuchFieldError8) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.FLOAT.ordinal()] = 7;
        }
        catch (NoSuchFieldError noSuchFieldError9) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.INT.ordinal()] = 4;
        }
        catch (NoSuchFieldError noSuchFieldError10) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.LONG.ordinal()] = 5;
        }
        catch (NoSuchFieldError noSuchFieldError11) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.NONE.ordinal()] = 10;
        }
        catch (NoSuchFieldError noSuchFieldError12) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.NULL.ordinal()] = 11;
        }
        catch (NoSuchFieldError noSuchFieldError13) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.OTHER.ordinal()] = 19;
        }
        catch (NoSuchFieldError noSuchFieldError14) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.PACKAGE.ordinal()] = 17;
        }
        catch (NoSuchFieldError noSuchFieldError15) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.SHORT.ordinal()] = 3;
        }
        catch (NoSuchFieldError noSuchFieldError16) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.TYPEVAR.ordinal()] = 15;
        }
        catch (NoSuchFieldError noSuchFieldError17) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.UNION.ordinal()] = 20;
        }
        catch (NoSuchFieldError noSuchFieldError18) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.VOID.ordinal()] = 9;
        }
        catch (NoSuchFieldError noSuchFieldError19) {}
        try {
            $switch_TABLE$javax$lang$model$type$TypeKind2[TypeKind.WILDCARD.ordinal()] = 16;
        }
        catch (NoSuchFieldError noSuchFieldError20) {}
        return Factory.$SWITCH_TABLE$javax$lang$model$type$TypeKind = $switch_TABLE$javax$lang$model$type$TypeKind2;
    }
}
